/*******************************************************************************
 * command.c
 * Command interpreter for network servers
 *
 * June 26, 2004
 *
 * Michael Siegenthaler
 * www.msigi.net
 *
 ******************************************************************************/

#include <string.h>

#include "command.h"

command_t* commandSet = 0;

/*******************************************************************************
 * Stores a pointer to a command set in a place where other functions of this
 * module can access it
 * Parameter: Pointer to a command set (array)
 ******************************************************************************/
void set_command_set(command_t* cmdset) {
  commandSet = cmdset;
}

/*******************************************************************************
 * Parses a command
 * Parameter: command as a null-terminated string without a newline
 * Returns a structure containing the numeric id of the command and a pointer
 * to it's parameter (this points to a location within the passed-in string,
 * or can be null if there are no parameters)
 ******************************************************************************/
parsed_command_t parse_command(char* cmdstr) {

  // Variables
  int i = 0;
  int length;
  parsed_command_t parsedCommand;

  // Initialize the parsed command to zero
  parsedCommand.cmdid = 0;
  parsedCommand.param = 0;

  // Look for a match in the command set
  while (1) {

    // Check if we've reached the end of the command set
    if ( (commandSet[i].cmdid == 0) || (commandSet[i].command == 0) ) {
      // Reached the end
      break;
    }

    length = strlen(commandSet[i].command);

    if (strncmp(commandSet[i].command, cmdstr, length) == 0) {
      int j;
      parsedCommand.cmdid = commandSet[i].cmdid;
      length = strlen(cmdstr);
      j = strlen(commandSet[i].command);
      while ( j < length ) {
        if (cmdstr[j] != ' ') {
          parsedCommand.param = cmdstr + j;
          // Break, causing parsed command to be returned
          break;
        }
        j++;
      }
      // Break, causing parsed command to be returned
      break;
    }

    i++;
  }

  // Return the parsed command
  return parsedCommand;

}
