/*----------------------------- gpibcomm.c ------------------------------------

  GPIB Driver Dummy File (for testing without a gpib card)

  14. August 2004

  by Michael Siegenthaler
     michael@msigi.net
     www.msigi.net

------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "ugpib.h"
#include "gpibcomm.h"

#define SMALL_DELAY 100*1000

// Simulation flags
int readerr = 0;
int readcount = 1;
double scale_factor = 1.0;

int address = 0;

int timeout_val = 3;

char lastwritten[1000];

/*******************************************************************************
  Initialize the GPIB Bus
*******************************************************************************/
int bus_init (void)
{
  lastwritten[0] = '\0';
  usleep(SMALL_DELAY);
  return 0;

}	/* bus_init() */



/*******************************************************************************
  Send a GPIB command
*******************************************************************************/
int gpib_write (int inst, char *cmd)
{
if (address == 12) {
  if ( !strcmp(cmd, "readerr") ) {
    readerr = 1;
  } else if ( !strcmp(cmd, "F1") ) {
    scale_factor = 1.0;
  } else if ( !strcmp(cmd, "F2") ) {
    scale_factor = -1.0;
  } else if ( !strcmp(cmd, "F3") ) {
    scale_factor = 100;
  } else if ( !strcmp(cmd, "F4") ) {
    scale_factor = 1000;
  } else if ( !strcmp(cmd, "F5") ) {
    scale_factor = 0.01;
  } else if ( !strcmp(cmd, "F6") ) {
    scale_factor = 0.001;
  } else if ( !strcmp(cmd, "badcommand") ) {
    return -1;
  }
  strncpy(lastwritten, cmd, 950);
  lastwritten[900] = '\0';
  usleep(SMALL_DELAY);
  return 0;
} else {
  if ( !strcmp(cmd, "readerr") ) {
    readerr = 1;
  } else if ( !strcmp(cmd, "badcommand") ) {
    return -1;
  }
  strncpy(lastwritten, cmd, 950);
  lastwritten[900] = '\0';
  usleep(SMALL_DELAY);
  return 0;
}

}	/* gpib_write() */


/*******************************************************************************
  Read from the GPIB bus
*******************************************************************************/
int gpib_read (int inst, char *buf, int buflen)
{
if (address == 12) {
  double random_number = drand48();
  random_number *= 12 * scale_factor;
  if (readerr) {
    readerr = 0;
    sleep(timeout_val);
    return -1;
  }
  snprintf(buf, buflen, "%e", random_number);
//  snprintf(buf, buflen, "seq=%d, last=%s", readcount, lastwritten);
//  strncpy(buf, "+3.14159265", buflen);
  readcount++;
  buf[buflen-1] = '\0';
  usleep(SMALL_DELAY);
  return 0;
} else {
  if (readerr) {
    readerr = 0;
    sleep(timeout_val);
    return -1;
  }
//  strncpy(buf, "+3.14159265", buflen);
  snprintf(buf, buflen, "seq=%d, last=%s", readcount, lastwritten);
  readcount++;
//  buf[buflen-1] = '\0';
  usleep(SMALL_DELAY);
  return 0;
}

}	/* gpib_read() */

/*******************************************************************************
  Initialize instrument
  Returns an instrument handle on success
*******************************************************************************/
int inst_init (int inst_addr)
{
  lastwritten[0] = '\0';
  usleep(SMALL_DELAY);
  if (inst_addr == 9 || inst_addr == 12) {
    address = inst_addr;
    return 1;
  } else {
    return -1;
  }
}	/* inst_init() */


/*******************************************************************************
  Close communications channel to instrument
*******************************************************************************/
int inst_term (int inst)
{
  usleep(SMALL_DELAY);
  return 0;
}	/* inst_term() */

/*******************************************************************************
  Set timeout value
*******************************************************************************/
int set_timeout_val (int tmoutval)
{
  if ( (tmoutval < 0) || (tmoutval > 17) ) {
    return -1;
  }
  timeout_val = tmoutval;
  return 0;
}	/* inst_term() */
