/*******************************************************************************
 * log.c
 * Logging capabilities for daemon processes
 *
 * June 26, 2004
 *
 * Michael Siegenthaler
 * www.msigi.net
 *
 ******************************************************************************/
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

#include "log.h"

FILE* logFile = 0; // file descriptor for log file

/*******************************************************************************
 * Open the log file and store the file descriptor
 * Parameter: Filename for the log
 * Returns 0 on success, or -1 on failure
 ******************************************************************************/
int openLogFile(char* logFileName) {

  logFile = 0;   // initialize pointer to be NULL

  logFile = fopen(logFileName, "a"); // open the
  if (!logFile) {
    return -1;
  }

  setlinebuf(logFile); // use line buffered mode

//  fprintf(logFile,"Opened logfile\n");

//  atexit(&closeLogFile); // close the logfile on exit

  return 0;
}

/*******************************************************************************
 * Write a string to the log
 ******************************************************************************/
void logMessage(char* msg) {

  sigset_t set;
  sigset_t oldset;

  if (logFile && msg) {

    // Block SIGTERM, because its handler might write to the logfile
    sigemptyset(&set);
    if ( sigaddset(&set, SIGTERM) ) {
      fprintf(logFile,"(cannot add SIGTERM to mask) ");
    }
    if ( sigprocmask(SIG_BLOCK, &set, &oldset) ) {
      fprintf(logFile,"(cannot block SIGTERM) ");
    }

    // Write message to log
    printTimeAndPid();
    fprintf(logFile, "%s\n", msg);

    // Unblock SIGTERM
    if ( sigprocmask(SIG_SETMASK, &oldset, NULL) ) {
      fprintf(logFile,"(cannot restore old signal set)\n");
    }

  }
}

/*******************************************************************************
 * Write a string to the log and then deallocate the string from memory
 ******************************************************************************/
void dLogMessage(char* msg) {
  if (msg) {
    logMessage(msg);
    free(msg);
  }
}

/*******************************************************************************
 * Get file stream for log file
 ******************************************************************************/
FILE* getLogFileStream() {
  return logFile;
}

/*******************************************************************************
 * Print time and process info to log file
 ******************************************************************************/
void printTimeAndPid() {

  struct tm* t = 0;
  time_t currentTime;
  pid_t myPID;

  myPID = getpid();

  currentTime = time(NULL);

  t = localtime(&currentTime);

  fprintf(logFile, "%4d-%02d-%02d %02d:%02d:%02d [%d] ",
           (t->tm_year)+1900,
           (t->tm_mon)+1,
           t->tm_mday,
           t->tm_hour,
           t->tm_min,
           t->tm_sec,
           myPID );

}

/*******************************************************************************
 * Close the log file
 ******************************************************************************/
void closeLogFile() {
  if (logFile) {
    //fprintf(logFile,"Closing logfile\n");
    fclose(logFile);
  }
}
