/*----------------------------- gpibcomm.c ------------------------------------

  GPIB Driver Generic Functions

  25. June 2004

  by Michael Siegenthaler
     michael@msigi.net
     www.msigi.net

------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "ugpib.h"
#include "gpibcomm.h"

int bus;			/* Communications channel for GPIB card */
int timeout_val = T10s;         /* Default timeout */


/*******************************************************************************
  Initialize the GPIB Bus
*******************************************************************************/
int bus_init (void)
{

  bus = ibfind (INTERFACE);
  if (ibsta & ERR) {
    return -1;
  }

  ibsic (bus);
  if (ibsta & ERR) {
    return -1;
  }

  return 0;

}	/* bus_init() */



/*******************************************************************************
  Send a GPIB command
*******************************************************************************/
int gpib_write (int inst, char *cmd)
{

  long length;

  length = strlen (cmd);

  ibwrt (inst, cmd, length);
  if (ibsta & ERR) {
    return -1;
  }

  return 0;

}	/* gpib_write() */


/*******************************************************************************
  Read from the GPIB bus
*******************************************************************************/
int gpib_read (int inst, char *buf, int buflen)
{

  int i = 0;

  ibrd (inst, buf, buflen);
  if (ibsta & ERR) {
    return -1;
  }

  buf[ibcntl] = '\0';

  while (buf[i] != '\0')
    {
      if (buf[i] == '\n')
	{
	  buf[i] = '\0';
	  break;
	}	/* if */
      i++;
    }	/* while */

  return 0;

}	/* gpib_read() */

/*******************************************************************************
  Initialize instrument
  Returns an instrument handle on success
*******************************************************************************/
int inst_init (int inst_addr)
{
  int inst;

  inst = ibdev (0, inst_addr, 0, timeout_val, 1, 0);
  if (ibsta & ERR)
    {
      return -1;
    }
sleep(5);

  ibclr (inst);
  if (ibsta & ERR)
    {
      return -1;
    }
sleep(5);
  return inst;
}	/* inst_init() */


/*******************************************************************************
  Close communications channel to instrument
*******************************************************************************/
int inst_term (int inst)
{
  ibonl (inst, 0);
  return 0;
}	/* inst_term() */


/*******************************************************************************
  Set timeout value
*******************************************************************************/
int set_timeout_val (int tmoutval)
{
  if ( (tmoutval < 0) || (tmoutval > 17) ) {
    return -1;
  }
  timeout_val = tmoutval;
  return 0;
}	/* inst_term() */
