/*******************************************************************************
 * start.c
 * Main program for running a daemon
 *
 * June 26, 2004
 *
 * Michael Siegenthaler
 * www.msigi.net
 *
 ******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "log.h"
#include "daemon.h"
#include "loadconfig.h"
#include "application.h"

#define CONFIGFILE "config.conf"

/*******************************************************************************
 * Main function
 ******************************************************************************/
int main (int argc, char **argv) {

  char* logFileName;
  char* configFileName;

  // Either get config file name from command line or use default
  if (argc == 2) {
    configFileName = argv[1];
  } else {
    configFileName = CONFIGFILE;
  }

  if ( loadConfig(configFileName) < 0 ) {
    fprintf(stderr, "FATAL: Cannot load config file: %s\n", configFileName);
    exit(EXIT_FAILURE);
  }

  //printArrayDebug();

  logFileName = getParamByName("logfile");
  if (!logFileName) {
    fprintf(stderr, "FATAL: Logfile not specified in config.\n");
    exit(EXIT_FAILURE);
  }

  if(openLogFile(logFileName)) {
    fprintf(stderr,"FATAL: Cannot open log: %s\n",logFileName);
    exit(EXIT_FAILURE);
  }

  // Turn the process into a daemon
  makeDaemon();

  // Run the application
  application();

  logMessage("Exiting because the application finished");

  exit(EXIT_SUCCESS);
  return 0;
}
